package java.nio;

import sun.misc.Unsafe;
import java.io.FileDescriptor;

public abstract class MappedByteBuffer extends ByteBuffer
{
    private final FileDescriptor fd;
    private static byte unused;
    
    MappedByteBuffer(int n, int n2, int n3, int n4, FileDescriptor fd) {
        super(n, n2, n3, n4);
        this.fd = fd;
    }
    
    MappedByteBuffer(int n, int n2, int n3, int n4) {
        super(n, n2, n3, n4);
        this.fd = null;
    }
    
    private void checkMapped() {
        if (this.fd == null) {
            throw new UnsupportedOperationException();
        }
        return;
    }
    
    private long mappingOffset() {
        int pageSize = Bits.pageSize();
        long n = this.address % (long)pageSize;
        return (n >= 0L) ? n : ((long)pageSize + n);
    }
    
    private long mappingAddress(long n) {
        return this.address - n;
    }
    
    private long mappingLength(long n) {
        return (long)this.capacity() + n;
    }
    
    public final boolean isLoaded() {
        this.checkMapped();
        if (this.address == 0L || this.capacity() == 0) {
            return true;
        }
        long mappingOffset = this.mappingOffset();
        long mappingLength = this.mappingLength(mappingOffset);
        return this.isLoaded0(this.mappingAddress(mappingOffset), mappingLength, Bits.pageCount(mappingLength));
    }
    
    public final MappedByteBuffer load() {
        this.checkMapped();
        if (this.address == 0L || this.capacity() == 0) {
            return this;
        }
        long mappingOffset = this.mappingOffset();
        long mappingLength = this.mappingLength(mappingOffset);
        this.load0(this.mappingAddress(mappingOffset), mappingLength);
        Unsafe unsafe = Unsafe.getUnsafe();
        int pageSize = Bits.pageSize();
        int pageCount = Bits.pageCount(mappingLength);
        long mappingAddress = this.mappingAddress(mappingOffset);
        int unused = 0;
        for (int i = 0; i < pageCount; ++i) {
            unused = (byte)(unused ^ unsafe.getByte(mappingAddress));
            mappingAddress += (long)pageSize;
        }
        if (MappedByteBuffer.unused != 0) {
            MappedByteBuffer.unused = unused;
            return this;
        }
        return this;
    }
    
    public final MappedByteBuffer force() {
        this.checkMapped();
        if (this.address != 0L && this.capacity() != 0) {
            long mappingOffset = this.mappingOffset();
            this.force0(this.fd, this.mappingAddress(mappingOffset), this.mappingLength(mappingOffset));
            return this;
        }
        return this;
    }
    
    private native boolean isLoaded0(long p0, long p1, int p2);
    
    private native void load0(long p0, long p1);
    
    private native void force0(FileDescriptor p0, long p1, long p2);
}
